@@ -25,15 +25,35 @@
 #include <Python.h>
 #include <numpy/ndarrayobject.h>
 #include <numpy/ndarraytypes.h>
+#include <math.h>
 
 #include <ORTHPOLxx/ORTHPOLPP.hpp>
 
-static PyObject *ORTHPOLError;
+using namespace std;
 
-/* Auxiliary functions declarations */
+static PyObject *ORTHPOLError;
+static PyObject *polymodError;
+
+/* .... C vector utility functions ..................*/
+PyArrayObject *pyvector(PyObject *objin);
 double *pyvector_to_Carrayptrs(PyArrayObject *arrayin);
 int  not_doublevector(PyArrayObject *vec);
 
+/* .... C matrix utility functions ..................*/
+PyArrayObject *pymatrix(PyObject *objin);
+double **pymatrix_to_Carrayptrs(PyArrayObject *arrayin);
+double **ptrvector(long n);
+void free_Carrayptrs(double **v);
+int  not_doublematrix(PyArrayObject *mat);
+
+/* .... C 2D int array utility functions ..................*/
+PyArrayObject *pyint2Darray(PyObject *objin);
+int **pyint2Darray_to_Carrayptrs(PyArrayObject *arrayin);
+int **ptrintvector(long n);
+void free_Cint2Darrayptrs(int **v);
+int  not_int2Darray(PyArrayObject *mat);
+
+
 /* ORTHPOL functions declaration */
 static PyObject* Py_r1mach (PyObject *self, PyObject *args);
 static PyObject* Py_d1mach (PyObject *self, PyObject *args);
@@ -48,6 +68,9 @@ static PyObject* Py_dmcdis (PyObject *self, PyObject *args);
 static PyObject* Py_dradau (PyObject *self, PyObject *args);
 static PyObject* Py_drecur (PyObject *self, PyObject *args);
 static PyObject* Py_dsti   (PyObject *self, PyObject *args);
+/* polymod functions declaration */
+static PyObject* Py_polyeval (PyObject *self, PyObject *args);
+static PyObject* Py_vandermonde (PyObject *self, PyObject *args);
 
 /* PyORTHPOL set callback functions declarations */
 static PyObject* set_dquad_callback(PyObject *self, PyObject *args);
@@ -78,9 +101,148 @@ static PyMethodDef ORTHPOLMethods[] = {
   {"dsti",   Py_dsti,   METH_VARARGS, "Applies the Stieltjes procedure."},
   {"set_dquad_callback", set_dquad_callback, METH_VARARGS, "Set the dquad callback function"},
   {"set_dwf_callback", set_dwf_callback, METH_VARARGS, "Set the dwf callback function"},
+  {"polyeval", Py_polyeval, METH_VARARGS, "Given a set of points and the N+1 recursion coefficients, evaluate the polynomial of order N, normalized."},
+  {"vandermonde", Py_vandermonde, METH_VARARGS, "Given a set of points and the N+1 recursion coefficients, evaluate the vandermonde matrix of order N, normalized."},
   {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
+/* #### Vector Utility functions ######################### */
+
+/* ==== Make a Python Array Obj. from a PyObject, ================
+     generates a double vector w/ contiguous memory which may be a new allocation if
+     the original was not a double type or contiguous 
+  !! Must DECREF the object returned from this routine unless it is returned to the
+     caller of this routines caller using return PyArray_Return(obj) or
+     PyArray_BuildValue with the "N" construct   !!!
+*/
+PyArrayObject *pyvector(PyObject *objin)  {
+	return (PyArrayObject *) PyArray_ContiguousFromObject(objin,
+		NPY_DOUBLE, 1,1);
+}
+/* ==== Create 1D Carray from PyArray ======================
+    Assumes PyArray is contiguous in memory.             */
+double *pyvector_to_Carrayptrs(PyArrayObject *arrayin)  {
+	int i,n;
+	
+	n=arrayin->dimensions[0];
+	return (double *) arrayin->data;  /* pointer to arrayin data as double */
+}
+/* ==== Check that PyArrayObject is a double (Float) type and a vector ==============
+    return 1 if an error and raise exception */ 
+int  not_doublevector(PyArrayObject *vec)  {
+	if (vec->descr->type_num != NPY_DOUBLE || vec->nd != 1)  {
+		PyErr_SetString(PyExc_ValueError,
+			"In not_doublevector: array must be of type Float and 1 dimensional (n).");
+		return 1;  }
+	return 0;
+}
+
+/* #### Matrix Utility functions ######################### */
+
+/* ==== Make a Python Array Obj. from a PyObject, ================
+     generates a double matrix w/ contiguous memory which may be a new allocation if
+     the original was not a double type or contiguous 
+  !! Must DECREF the object returned from this routine unless it is returned to the
+     caller of this routines caller using return PyArray_Return(obj) or
+     PyArray_BuildValue with the "N" construct   !!!
+*/
+PyArrayObject *pymatrix(PyObject *objin)  {
+	return (PyArrayObject *) PyArray_ContiguousFromObject(objin,
+		NPY_DOUBLE, 2,2);
+}
+/* ==== Create Carray from PyArray ======================
+    Assumes PyArray is contiguous in memory.
+    Memory is allocated!                                    */
+double **pymatrix_to_Carrayptrs(PyArrayObject *arrayin)  {
+	double **c, *a;
+	int i,n,m;
+	
+	n=arrayin->dimensions[0];
+	m=arrayin->dimensions[1];
+	c=ptrvector(n);
+	a=(double *) arrayin->data;  /* pointer to arrayin data as double */
+	for ( i=0; i<n; i++)  {
+		c[i]=a+i*m;  }
+	return c;
+}
+/* ==== Allocate a double *vector (vec of pointers) ======================
+    Memory is Allocated!  See void free_Carray(double ** )                  */
+double **ptrvector(long n)  {
+	double **v;
+	v=(double **)malloc((size_t) (n*sizeof(double)));
+	if (!v)   {
+		printf("In **ptrvector. Allocation of memory for double array failed.");
+		exit(0);  }
+	return v;
+}
+/* ==== Free a double *vector (vec of pointers) ========================== */ 
+void free_Carrayptrs(double **v)  {
+	free((char*) v);
+}
+/* ==== Check that PyArrayObject is a double (Float) type and a matrix ==============
+    return 1 if an error and raise exception */ 
+int  not_doublematrix(PyArrayObject *mat)  {
+	if (mat->descr->type_num != NPY_DOUBLE || mat->nd != 2)  {
+		PyErr_SetString(PyExc_ValueError,
+			"In not_doublematrix: array must be of type Float and 2 dimensional (n x m).");
+		return 1;  }
+	return 0;
+}
+
+/* #### Integer Array Utility functions ######################### */
+
+/* ==== Make a Python int Array Obj. from a PyObject, ================
+     generates a 2D integer array w/ contiguous memory which may be a new allocation if
+     the original was not an integer type or contiguous 
+  !! Must DECREF the object returned from this routine unless it is returned to the
+     caller of this routines caller using return PyArray_Return(obj) or
+     PyArray_BuildValue with the "N" construct   !!!
+*/
+PyArrayObject *pyint2Darray(PyObject *objin)  {
+	return (PyArrayObject *) PyArray_ContiguousFromObject(objin,
+		NPY_LONG, 2,2);
+}
+/* ==== Create integer 2D Carray from PyArray ======================
+    Assumes PyArray is contiguous in memory.
+    Memory is allocated!                                    */
+int **pyint2Darray_to_Carrayptrs(PyArrayObject *arrayin)  {
+	int **c, *a;
+	int i,n,m;
+	
+	n=arrayin->dimensions[0];
+	m=arrayin->dimensions[1];
+	c=ptrintvector(n);
+	a=(int *) arrayin->data;  /* pointer to arrayin data as int */
+	for ( i=0; i<n; i++)  {
+		c[i]=a+i*m;  }
+	return c;
+}
+/* ==== Allocate a a *int (vec of pointers) ======================
+    Memory is Allocated!  See void free_Carray(int ** )                  */
+int **ptrintvector(long n)  {
+	int **v;
+	v=(int **)malloc((size_t) (n*sizeof(int)));
+	if (!v)   {
+		printf("In **ptrintvector. Allocation of memory for int array failed.");
+		exit(0);  }
+	return v;
+}
+/* ==== Free an int *vector (vec of pointers) ========================== */ 
+void free_Cint2Darrayptrs(int **v)  {
+	free((char*) v);
+}
+/* ==== Check that PyArrayObject is an int (integer) type and a 2D array ==============
+    return 1 if an error and raise exception
+    Note:  Use NY_LONG for NumPy integer array, not NP_INT      */ 
+int  not_int2Darray(PyArrayObject *mat)  {
+	if (mat->descr->type_num != NPY_LONG || mat->nd != 2)  {
+		PyErr_SetString(PyExc_ValueError,
+			"In not_int2Darray: array must be of type int and 2 dimensional (n x m).");
+		return 1;  }
+	return 0;
+}
+
+
 /* Init Package function definition */
 PyMODINIT_FUNC
 initorthpol(void)
@@ -93,10 +255,14 @@ initorthpol(void)
 
     import_array();
 
-    ORTHPOLError = PyErr_NewException("orthpol.error", NULL, NULL);
+    ORTHPOLError = PyErr_NewException(const_cast<char *>("orthpol.error"), NULL, NULL);
     Py_INCREF(ORTHPOLError);
     PyModule_AddObject(m, "error", ORTHPOLError);
 
+    polymodError = PyErr_NewException(const_cast<char *>("polymod.error"), NULL, NULL);
+    Py_INCREF(polymodError);
+    PyModule_AddObject(m, "error", polymodError);
+
     // Set callbacks functions for orthpol package
     orthpol::set_dquad_callback(dquad_callback_func);
     orthpol::set_dwf_callback(dwf_callback_func);
@@ -177,7 +343,7 @@ Py_dcheb(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dcheb(n, a, b, fnu, alpha, beta, s, ierr, s0, s1, s2);
 
-  return Py_BuildValue("OOOi", py_alpha, py_beta, py_s, ierr);
+  return Py_BuildValue("NNNi", py_alpha, py_beta, py_s, ierr);
 }
 
 static PyObject *
@@ -219,7 +385,7 @@ Py_dchri(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dchri(n, iopt, a, b, x, y, hr, hi, alpha, beta, ierr);
 
-  return Py_BuildValue("OOi", py_alpha, py_beta, ierr);
+  return Py_BuildValue("NNi", py_alpha, py_beta, ierr);
 }
 
 static PyObject *
@@ -264,7 +430,7 @@ Py_dgauss(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dgauss(n, a, b, eps, zero, weight, ierr, e);
 
-  return Py_BuildValue("OOi", py_zero, py_weight, ierr);
+  return Py_BuildValue("NNi", py_zero, py_weight, ierr);
 }
 
 static PyObject *
@@ -324,7 +490,7 @@ Py_dgchri(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dgchri(n, iopt, nu0, numax, eps, a, b, x, y, alpha, beta, nu, ierr, ierrc, fnu, s, s0, s1, s2);
 
-  return Py_BuildValue("OOiii", py_alpha, py_beta, nu, ierr, ierrc);
+  return Py_BuildValue("NNiii", py_alpha, py_beta, nu, ierr, ierrc);
 }
 
 static PyObject *
@@ -371,7 +537,7 @@ Py_dlancz(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dlancz(n, ncap, x, y, alpha, beta, ierr, p0, p1);
 
-  return Py_BuildValue("OOi", py_alpha, py_beta, ierr);
+  return Py_BuildValue("NNi", py_alpha, py_beta, ierr);
 }
 
 static PyObject *
@@ -420,7 +586,7 @@ Py_dlob(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dlob(n, alpha, beta, aleft, right, zero, weight, ierr, e, a, b);
 
-  return Py_BuildValue("OOi", py_zero, py_weight, ierr);
+  return Py_BuildValue("NNi", py_zero, py_weight, ierr);
 }
 
 static PyObject *
@@ -515,7 +681,7 @@ Py_dmcheb(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dmcheb(n, ncapm, mc, mp, xp, yp, eps, iq, idelta, finl, finr, endl, endr, xfer, wfer, a, b, fnu, alpha, beta, ncap, kount, ierr, be, x, w, xm, wm, s, s0, s1, s2);
 
-  return Py_BuildValue("OOiii", py_alpha, py_beta, ncap, kount, ierr);
+  return Py_BuildValue("NNiii", py_alpha, py_beta, ncap, kount, ierr);
 }
 
 static PyObject *
@@ -596,7 +762,7 @@ Py_dmcdis(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dmcdis(n, ncapm, mc, mp, xp, yp, eps, iq, idelta, irout, finl, finr, endl, endr, xfer, wfer, alpha, beta, ncap, kount, ierr, ie, be, x, w, xm, wm, p0, p1, p2);
 
-  return Py_BuildValue("OOiiii", py_alpha, py_beta, ncap, kount, ierr, ie);
+  return Py_BuildValue("NNiiii", py_alpha, py_beta, ncap, kount, ierr, ie);
 }
 
 static PyObject *
@@ -645,7 +811,7 @@ Py_dradau(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dradau(n, alpha, beta, end, zero, weight, ierr, e, a, b);
 
-  return Py_BuildValue("OOi", py_zero, py_weight, ierr);
+  return Py_BuildValue("NNi", py_zero, py_weight, ierr);
 }
 
 static PyObject *
@@ -674,7 +840,7 @@ Py_drecur(PyObject *self, PyObject *args)
   
   orthpol::drecur(n, ipoly, al, be, a, b, ierr);
 
-  return Py_BuildValue("OOi", alpha, beta, ierr);
+  return Py_BuildValue("NNi", alpha, beta, ierr);
 }
 
 static PyObject *
@@ -723,25 +889,7 @@ Py_dsti(PyObject *self, PyObject *args)
   // Call ORTHPOL library function
   orthpol::dsti(n, ncap, x, y, alpha, beta, ierr, p0, p1, p2);
 
-  return Py_BuildValue("OOi", py_alpha, py_beta, ierr);
-}
-
-/* Auxiliary functions declarations */
-
-/* ==== Create 1D Carray from PyArray ======================
-     Assumes PyArray is contiguous in memory.             */
-double *pyvector_to_Carrayptrs(PyArrayObject *arrayin)  {
-  return (double *) arrayin->data;  /* pointer to arrayin data as double */
-}
-
-/* ==== Check that PyArrayObject is a double (Float) type and a vector ==============
-   return 1 if an error and raise exception */ 
-int  not_doublevector(PyArrayObject *vec)  {
-  if (vec->descr->type_num != NPY_DOUBLE || vec->nd != 1)  {
-    PyErr_SetString(PyExc_ValueError,
-		    "In not_doublevector: array must be of type Float and 1 dimensional (n).");
-    return 1;  }
-  return 0;
+  return Py_BuildValue("NNi", py_alpha, py_beta, ierr);
 }
 
 /* PyORTHPOL set callback functions declarations */
@@ -843,14 +991,230 @@ double dwf_callback_func(double& x, int& i){
   // Retrive output and set values
   if (returnlist == NULL) {
     PyErr_SetString(PyExc_TypeError, "Null value returned by user defined function.");
-    return NULL;
+    return 0.;
   }
   if (!PyArg_ParseTuple(Py_BuildValue("(O)",returnlist), "d", &w)){
     PyErr_SetString(PyExc_TypeError, "Wrong type returned by user defined function.");
-    return NULL;
+    return 0.;
   }
   Py_DECREF(returnlist);
 
   // RETURN
   return w;
 }
+
+
+/* polymod functions definitions */
+static PyObject*
+Py_polyeval(PyObject *self, PyObject *args)
+{
+  /* VARIABLES */
+  // INPUT
+  int n;
+  PyArrayObject *py_alpha, *py_beta; // Evaluation points
+  PyArrayObject *py_rs; 	// Evaluation points
+  double *alpha, *beta, *rs;
+  // OUTPUT
+  PyArrayObject *py_out;
+  double *out;
+  // INTERNAL
+  int *dims = new int[1];
+  double *old1, *old2, *outinner, *tmp, *nrmpol, *nrmold1, *nrmold2;
+  double *zeros, *weights, *e;
+
+  if (!PyArg_ParseTuple(args, "O!iO!O!", &PyArray_Type, &py_rs, &n, &PyArray_Type, &py_alpha, &PyArray_Type, &py_beta))
+    return NULL;
+  if (NULL == py_rs) return NULL;
+  if (NULL == py_alpha) return NULL;
+  if (NULL == py_beta) return NULL;
+  
+  if (n < 0)
+    PyErr_SetString(polymodError, "The order must be bigger than 0.");
+  if ((PyArray_NDIM(py_rs) > 1) || (PyArray_NDIM(py_alpha) > 1) || (PyArray_NDIM(py_beta) > 1))
+    PyErr_SetString(polymodError, "The array provided must be 1 dimensional.");
+  if (( PyArray_SHAPE(py_alpha)[0] < n ) || ( PyArray_SHAPE(py_beta)[0] < n ))
+    PyErr_SetString(polymodError, "The recurrence coefficeints must be more than n.");
+  
+  dims[0] = PyArray_SHAPE(py_rs)[0];
+
+  // PyArray to C
+  rs = pyvector_to_Carrayptrs(py_rs);
+  alpha = pyvector_to_Carrayptrs(py_alpha);
+  beta = pyvector_to_Carrayptrs(py_beta);
+
+  // Allocate output variables
+  py_out = (PyArrayObject*) PyArray_FromDims(1, dims, NPY_DOUBLE);
+  out = pyvector_to_Carrayptrs(py_out);
+  // Allocate auxiliary space
+  outinner = new double[dims[0]];
+
+  // Internal normalization variables
+  int nrmpts = n+1;
+  zeros = new double[nrmpts];
+  weights = new double[nrmpts];
+  e = new double[nrmpts];
+  double eps = 1e-13;
+  int ierr;
+  // Call ORTHPOL library function
+  orthpol::dgauss(nrmpts, alpha, beta, eps, zeros, weights, ierr, e);
+  if (ierr != 0)
+    PyErr_SetString(polymodError, "Error in the generation of the Gauss points.");
+  delete [] e;
+  // Allocate normalization polynomial
+  nrmpol = new double[nrmpts];
+  
+  // Use three term recursion to compute the unnormalized polynomial
+  // and the normalizing polynomial
+  if (n >= 0) {
+    for (int i = 0; i < dims[0]; i++) outinner[i] = 1.;
+    for (int i = 0; i < nrmpts; i++) nrmpol[i] = 1.;
+  }
+  if (n >= 1) {
+    old1 = outinner;		// Copy pointer
+    nrmold1 = nrmpol;		// Copy pointer
+    outinner = new double[dims[0]];
+    nrmpol = new double[nrmpts];
+    for (int i = 0; i < dims[0]; i++) outinner[i] = rs[i] - alpha[0];
+    for (int i = 0; i < nrmpts; i++) nrmpol[i] = zeros[i] - alpha[0];
+  }
+  if (n >= 2) {
+    old2 = new double[dims[0]];
+    nrmold2 = new double[nrmpts];
+  }
+  for (int j=2; j <= n; j++){
+    // Swap pointers
+    tmp = old2;
+    old2 = old1;
+    old1 = outinner;
+    outinner = tmp;		// Here we put the new values (save in allocation)
+    // Swap pointers
+    tmp = nrmold2;
+    nrmold2 = nrmold1;
+    nrmold1 = nrmpol;
+    nrmpol = tmp;
+    for (int i = 0; i < dims[0]; i++)
+      outinner[i] = (rs[i] - alpha[j-1]) * old1[i] - beta[j-1] * old2[i];
+    for (int i = 0; i < nrmpts; i++)
+      nrmpol[i] = (zeros[i] - alpha[j-1]) * nrmold1[i] - beta[j-1] * nrmold2[i];
+  }
+
+  // Normalize
+  double sqnrm = 0.;
+  for (int i = 0; i < nrmpts; i++)
+    sqnrm += nrmpol[i] * nrmpol[i] * weights[i];
+  sqnrm = sqrt(sqnrm);
+  if (sqnrm == 0.)
+    PyErr_SetString(polymodError, "Division by zero");
+  for (int i = 0; i < dims[0]; i++)
+    outinner[i] /= sqnrm;
+
+  // Free space and prepare output
+  if (n>=1) { delete [] old1; delete [] nrmold1; }
+  if (n>=2) { delete [] old2; delete [] nrmold2; }
+  delete [] nrmpol;
+  delete [] zeros;
+  delete [] weights;
+  
+  memcpy( out, outinner, dims[0]*sizeof(double) );
+
+  delete [] outinner;
+  
+  return Py_BuildValue("N", py_out);
+}
+
+static PyObject*
+Py_vandermonde(PyObject *self, PyObject *args)
+{
+  /* VARIABLES */
+  // INPUT
+  int n;
+  PyArrayObject *py_alpha, *py_beta; // Evaluation points
+  PyArrayObject *py_rs; 	// Evaluation points
+  double *alpha, *beta, *rs;
+  // OUTPUT
+  PyArrayObject *py_out;
+  double **out;
+  // INTERNAL
+  int *dims = new int[2];
+  double **nrmpol;
+  double *zeros, *weights, *e;
+
+  if (!PyArg_ParseTuple(args, "O!iO!O!", &PyArray_Type, &py_rs, &n, &PyArray_Type, &py_alpha, &PyArray_Type, &py_beta))
+    return NULL;
+  if (NULL == py_rs) return NULL;
+  if (NULL == py_alpha) return NULL;
+  if (NULL == py_beta) return NULL;
+  
+  if (n < 0)
+    PyErr_SetString(polymodError, "The order must be bigger than 0.");
+  if ((PyArray_NDIM(py_rs) > 1) || (PyArray_NDIM(py_alpha) > 1) || (PyArray_NDIM(py_beta) > 1))
+    PyErr_SetString(polymodError, "The array provided must be 1 dimensional.");
+  if (( PyArray_SHAPE(py_alpha)[0] < n ) || ( PyArray_SHAPE(py_beta)[0] < n ))
+    PyErr_SetString(polymodError, "The recurrence coefficeints must be more than n.");
+  
+  dims[0] = PyArray_SHAPE(py_rs)[0];
+  dims[1] = n+1;
+
+  // PyArray to C
+  rs = pyvector_to_Carrayptrs(py_rs);
+  alpha = pyvector_to_Carrayptrs(py_alpha);
+  beta = pyvector_to_Carrayptrs(py_beta);
+
+  // Allocate output variables
+  py_out = (PyArrayObject*) PyArray_FromDims(2, dims, NPY_DOUBLE);
+  out = pymatrix_to_Carrayptrs(py_out);
+
+  // Internal normalization variables
+  int nrmpts = n+1;
+  zeros = new double[nrmpts];
+  weights = new double[nrmpts];
+  e = new double[nrmpts];
+  double eps = 1e-13;
+  int ierr;
+  // Call ORTHPOL library function
+  orthpol::dgauss(nrmpts, alpha, beta, eps, zeros, weights, ierr, e);
+  if (ierr != 0)
+    PyErr_SetString(polymodError, "Error in the generation of the Gauss points.");
+  delete [] e;
+  // Allocate normalization polynomial
+  nrmpol = new double*[nrmpts];
+  for (int i = 0; i < nrmpts; i++)
+    nrmpol[i] = new double[nrmpts];
+  
+  // Use three term recursion to compute the unnormalized polynomial
+  // and the normalizing polynomial
+  if (n >= 0) {
+    for (int i = 0; i < dims[0]; i++) out[i][0] = 1.;
+    for (int i = 0; i < nrmpts; i++) nrmpol[i][0] = 1.;
+  }
+  if (n >= 1) {
+    for (int i = 0; i < dims[0]; i++) out[i][1] = rs[i] - alpha[0];
+    for (int i = 0; i < nrmpts; i++) nrmpol[i][1] = zeros[i] - alpha[0];
+  }
+  for (int i = 0; i < dims[0]; i++)
+    for (int j=2; j <= n; j++)
+      out[i][j] = (rs[i] - alpha[j-1]) * out[i][j-1] - beta[j-1] * out[i][j-2];
+  for (int i = 0; i < nrmpts; i++)
+    for (int j=2; j <= n; j++)
+      nrmpol[i][j] = (zeros[i] - alpha[j-1]) * nrmpol[i][j-1] - beta[j-1] * nrmpol[i][j-2];
+
+  // Normalize
+  double *sqnrm = new double[nrmpts];
+  for (int j = 0; j < nrmpts; j++) sqnrm[j] = 0.;
+  for (int i = 0; i < nrmpts; i++){
+    for (int j = 0; j < nrmpts; j++)
+      sqnrm[j] += nrmpol[i][j] * nrmpol[i][j] * weights[i];
+  }
+  for (int j = 0; j < nrmpts; j++) sqnrm[j] = sqrt(sqnrm[j]);
+  for (int i = 0; i < dims[0]; i++)
+    for (int j = 0; j < nrmpts; j++)
+      out[i][j] /= sqnrm[j];
+
+  // Free space and prepare output
+  delete [] nrmpol;
+  delete [] zeros;
+  delete [] weights;
+  free_Carrayptrs(out);
+  
+  return PyArray_Return(py_out);
+}
